Player = {}

function Player:load()

    self.x = love.graphics.getWidth() / 2
    self.y = love.graphics.getHeight() / 2 - 62
    self.spriteSheet = love.graphics.newImage("Sprites/TheWizzl.png")
    self.speed = 400

    self.ground = self.y
    self.yVel = 0
    self.jump_height = -300
    self.gravity = -500

    -- Animation Information

    local grid = anim8.newGrid( -- No need to store this function in the table
        32,32, self.spriteSheet:getWidth(), self.spriteSheet:getHeight()
    )

    self.anim = {}

    self.anim.idle = anim8.newAnimation(grid('1-4', 1), 0.2)
    self.anim.hat = anim8.newAnimation(grid(28, 1), 0.1)

    self.anim.run = anim8.newAnimation(grid('8-11', 1), 0.1)
    self.anim.hatWalk = anim8.newAnimation(grid('29-33', 1), 0.1)

    self.anim.jump = anim8.newAnimation(grid(19, 1), 0.2)
    self.anim.fall = anim8.newAnimation(grid(20, 1), 0.2)

    self.anim.hide = anim8.newAnimation(grid('22-27', 1), 0.1, function ()
        self:setState("hat")    -- Applies the "hat" state at the end of the animation
        self.on_anim = false
    end)

    self.anim.getUp = anim8.newAnimation(grid('35-38', 1), 0.1, function ()
        self:setState("idle")   -- Applies the "idle" state at the end of the animation
        self.on_anim = false
    end)

    self:setState("idle")
    self.on_anim = false    -- This variable will be used to not launch a walk animation while another is in progress (jump, fall, hide, getUp)
    self.crouch = false
    self.switch = 2

end

function Player:setState(state)

    if state ~= self.state then

        self.anim_curr = self.anim[state]
        self.anim_curr:gotoFrame(1)

        self.prev_state = self.state
        self.state = state

    end

end

function Player:update(dt)

    if love.keyboard.isScancodeDown("s") then

        if not self.crouch
        and self.y == self.ground then  -- To avoid squatting in the air
            self:setState("hide")
            self.on_anim = true
            self.crouch = true
        end

    end

    if love.keyboard.isScancodeDown("space") then

        if self.crouch then
            self:setState("getUp")
            self.on_anim = true
            self.crouch = false
        end

    end

    if love.keyboard.isScancodeDown("escape") then
        love.event.quit()
    end

    -- Movement and anim update --

    Player:move(dt)
    Player:jump(dt)

    self.anim_curr:update(dt)

end

function Player:draw()
    self.anim_curr:draw(self.spriteSheet, self.x, self.y, nil, self.switch, 2, 16)
end

function Player:move(dt)

    local onMove;

    if love.keyboard.isScancodeDown("a") then
        self.x = self.x - self.speed * dt
        self.switch = -2
        onMove = true

    elseif love.keyboard.isScancodeDown("d") then
        self.x = self.x + self.speed * dt
        self.switch = 2
        onMove = true

    end

    if not self.on_anim then

        if onMove then
            self:setState((self.crouch) and "hatWalk" or "run")
        else
            self:setState((self.crouch) and "hat" or "idle")
        end

    end

end

function Player:jump(dt)

    if self.crouch == false then

        if love.keyboard.isScancodeDown("w") then

            if self.yVel == 0 then              -- Start jump action
                self.yVel = self.jump_height
                self:setState("jump")
                self.on_anim = true     -- To prevent the animation from changing in `Player:move`
            end

        end

        if self.yVel ~= 0 then                  -- Perform the jumping movement
            self.y = self.y + self.yVel * dt
            self.yVel = self.yVel - self.gravity * dt
            if self.yVel > 0 then self:setState("fall") end
        end

        if self.y > self.ground then            -- Check if you touch the ground
            self:setState("idle")
            self.on_anim = false        -- Animation can change again in `Player:move`
            self.y = self.ground
            self.yVel = 0
        end

    end

end

